﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using Twp.Utilities;
using Twp.Controls;
using FCGame;

namespace AOcrayon
{
    public class GameModeManager //: IGameModeHost
    {
        public GameModeManager()
        {
        }

        private GameModeCollection modes = new GameModeCollection();
        public GameModeCollection Modes
        {
            get { return this.modes; }
            set { this.modes = value; }
        }

        public void FindModes()
        {
            this.FindModes( AppDomain.CurrentDomain.BaseDirectory );
        }

        public void FindModes( string path )
        {
            this.modes.Clear();

            foreach( string fileName in Directory.GetFiles( path, "FCGame.*.dll" ) )
            {
                this.AddMode( fileName );
            }
        }

        public void CloseGameModes()
        {
            foreach( IGameMode mode in this.modes )
            {
                mode.Dispose();
            }
            this.modes.Clear();
        }

        public void AddMode( string fileName )
        {
            Assembly asm = Assembly.LoadFrom( fileName );
            foreach( Type type in asm.GetTypes() )
            {
                if( type.IsPublic && !type.IsAbstract )
                {
                    Type gmInterface = type.GetInterface( "FCGame.IGameMode", true );
                    if( gmInterface != null )
                    {
                        IGameMode mode = (IGameMode) Activator.CreateInstance( asm.GetType( type.ToString() ) );
                        Log.Debug( "[GameModeManager.AddMode] Mode: {0}", mode.Name );
                        this.modes.Add( mode );
                    }
                }
            }
        }
    }
}
